; Player/Missile demo #1
; Charles Wieland, 6/23/2014
; junkosoft.com

	processor 6502
	include vcs.h

; Constant values
MINHORZPOS = $18
MAXHORZPOS = $9E
CENTERPOS = $5B

; Labels for RAM
; Horizontal position of the enemy sprite
ENEMYHORZPOS = $80

; Vertical position of the enemy sprite
ENEMYVERTPOS = $81

; Offset between current scan line and the render location of the enemy sprite
ENEMYVERTPOSTEMP = $83    

; Color of enemy ship
ENEMYCOLOR = $84

; Horizontal position of the player sprite
PLAYERHORZPOS = $85

; Vertical position of the player's missile
MISSILEVERTPOS = $86

; Offset between current scan line and the render location of the player's missile
MISSILEVERTPOSTEMP = $87

; Horizontal position of the player's missile
MISSILEHORZPOS = $88

; $00 when the button is released, $01 when the button is being held
BUTTONDOWN = $89

; Shooting sound effect counter
SHOOTSOUNDCOUNTER = $8A

; Score, number of times enemy hit
SCORE = $8B

; Number of frames when enemy is hit
DEATHFRAMECOUNT = $8C

; Incremented for each frame, may be used for random number generation later??
FRAMECOUNT = $90

	ORG $F000
Start	
; Initialize the system
	SEI
	CLD
	LDX #$FF
	TXS
	LDA #$00

ClearStack
	STA $00,X
	DEX
	BNE ClearStack

; Define initial values
	LDA #CENTERPOS
	STA PLAYERHORZPOS
	LDA #$2E
	STA COLUP0
	LDA #$01
	STA CTRLPF
	LDA #$05
	STA AUDC0
	LDA #$03
	STA AUDC1
	LDA #$30
	STA ENEMYVERTPOS
	LDA #MINHORZPOS
	STA ENEMYHORZPOS
	
; Start vertical blank	
DrawVBlank
	LDA #$02
	STA VSYNC
	STA WSYNC
	STA WSYNC
	STA WSYNC
	LDA #$00
	STA VSYNC
	STA VBLANK

; Set the timer
	LDA #$2C
	STA TIM64T

; Increment frame count and get the current ship color
	INC FRAMECOUNT
	LDA DEATHFRAMECOUNT
	STA AUDV1
	BEQ SetShipColor
	
	STA COLUP1
	STA AUDF1
	DEC DEATHFRAMECOUNT
	
	BEQ NextEnemy
	JMP CheckButton	

SetShipColor	
	LDX ENEMYCOLOR
	LDA ShipColors,X
	STA COLUP1

; Handle collisions
	LDA CXM0P
	AND #$80
	BEQ MoveEnemy
	
	LDA #$00
	STA MISSILEVERTPOS
	INC SCORE
	
	LDA #$0F
	STA DEATHFRAMECOUNT

MoveEnemy
	LDA #MAXHORZPOS
	CMP ENEMYHORZPOS
	BEQ ResetEnemy
	INC ENEMYHORZPOS
	JMP CheckButton

NextEnemy
	INC ENEMYCOLOR
	LDA ENEMYCOLOR
	AND #$03
	STA ENEMYCOLOR
	
ResetEnemy
	LDA #MINHORZPOS
	STA ENEMYHORZPOS
	
CheckButton
	STA CXCLR
	LDA INPT4
    BMI ClearButtonFlag
    
	LDA BUTTONDOWN
	BNE CheckJoystick	

SetButtonFlag
; Flag button as pressed
	LDA #$01
	STA BUTTONDOWN
	
; Make sure a missile isn't launched
	LDA MISSILEVERTPOS
	BNE CheckJoystick
	
; Copy current player horizontal position into missile horizontal position, adjusting to center missile over player
	LDA PLAYERHORZPOS
	ADC #$04
	STA MISSILEHORZPOS
	
; Set missile's vertical position slightly above player
	LDA #$A4
	STA MISSILEVERTPOS
	
; Set sound effect counter
	LDA #$E0
	STA SHOOTSOUNDCOUNTER

ClearButtonFlag
; Button released
	LDA #$00
	STA BUTTONDOWN
	
CheckJoystick
	LDA #$40
	BIT SWCHA		; Left
	BEQ MovePlayerLeft
	
	LDA #$80
	BIT SWCHA		; Right
	BEQ MovePlayerRight
	JMP PositionObjects
	
MovePlayerLeft
	LDA PLAYERHORZPOS
	CMP #MINHORZPOS
	BEQ PositionObjects
	
	DEC PLAYERHORZPOS
	JMP PositionObjects

MovePlayerRight
	LDA PLAYERHORZPOS
	CMP #MAXHORZPOS
	BEQ PositionObjects

	INC PLAYERHORZPOS
	
PositionObjects	
; Position the missile
	LDX #$02
	LDA MISSILEHORZPOS
	JSR PositionObject
	STA WSYNC

; Position the player
	LDX #$00
	LDA PLAYERHORZPOS
	JSR PositionObject
	STA WSYNC
	
; Position the enemy
	LDX #$01
	LDA ENEMYHORZPOS
	JSR PositionObject
	STA WSYNC
	
; Do the sound effect
	LDA SHOOTSOUNDCOUNTER
	BEQ DisableSound
	
	INC SHOOTSOUNDCOUNTER
	STA AUDF0
	LDA #$0F
	
DisableSound
	STA AUDV0
	
; Copy vertical positions into temp locations
CopyValue
	LDA ENEMYVERTPOS
	STA ENEMYVERTPOSTEMP
	LDA MISSILEVERTPOS
	STA MISSILEVERTPOSTEMP

WaitForTimer
	LDA INTIM
	BNE WaitForTimer
	STA WSYNC
    STA VBLANK

; Define the playfield
	LDY #$80
	LDA #$FF
	STA PF0
	STA PF1
	STA PF2

;  Draws 8 lines of blue border
DrawTopArea
	STY COLUPF
	STA WSYNC
	INY
	INY
	CPY #$90
	BNE DrawTopArea
	
	LDA #$30						; 2
	STA PF0							; 3
	LDA #$00						; 2
	STA PF1							; 2
	STA PF2							; 2

; Draw 168 lines of the play area
	LDY #$A8			         	; 2
	
DrawPlayArea
	LDA #$F8						; 2
	DEC MISSILEVERTPOSTEMP			; 3
	DEC ENEMYVERTPOSTEMP			; 3
	STA WSYNC						; 3

	BIT ENEMYVERTPOSTEMP			; 3
	BNE CheckMissilePosition		; 2

	LDX ENEMYVERTPOSTEMP			; 3
	LDA Ship,X						; 3
	STA GRP1						; 3

CheckMissilePosition
	LDA #$FC						; 2
	BIT MISSILEVERTPOSTEMP			; 3
	BNE FinishScanline				; 2
	
	LDA #$02						; 2

FinishScanline
	STA ENAM0						; 3
	DEY								; 2
	BNE DrawPlayArea				; 2
	
	; Draw player, 8 lines
	LDX #$07
DrawPlayer
	STA WSYNC
	LDA P0Data,X
	STA GRP0
	DEX
	BPL DrawPlayer

;  Draws 8 lines of blue border
	LDY #$8E
	LDA #$FF
	STA PF0
	STA PF1
	STA PF2
	INX
	STX GRP0
	STX ENAM0

DrawBottom
	STY COLUPF
	STA WSYNC
	DEY
	DEY
	CPY #$7E
	BNE DrawBottom
	
	LDA #$00
	STA PF0
	STA PF1
	STA PF2

; Check position of missile
	LDA MISSILEVERTPOS
	BEQ DrawOverscan
	
; Change vertical position of missile
	DEC MISSILEVERTPOS
	DEC MISSILEVERTPOS
	LDA #$02
	STA VBLANK
	
; DrawOverscan (30 scanlines)
DrawOverscan
	LDY #$1E
DrawOverscanline
    STA WSYNC
	DEY
	BNE DrawOverscanline

	LDA #$02
	STA VBLANK
	JMP DrawVBlank

PositionObject:
	STA HMCLR
	SEC
	STA WSYNC

WaitForPosition:
	SBC #$0F
	BCS WaitForPosition

	EOR #$07
	ASL
	ASL
	ASL
	ASL
	STA RESP0,X
	STA HMP0,X
	STA WSYNC
	STA HMOVE
	RTS
	
Ship
	.byte %00000000
	.byte %00011000
	.byte %00011000
	.byte %11111111
	.byte %00000000
	.byte %11111111
	.byte %00011000
	.byte %00011000
	
ShipColors
	.byte $44
	.byte $66
	.byte $88
	.byte $CC
	
P0Data
	.byte %11100111
	.byte %11100111
	.byte %11100111
	.byte %01111110
	.byte %00111100
	.byte %00111100
	.byte %00011000
	.byte %00011000
	
	org $FFEB
	dc.b "(c)2014 JunkoSoft"
	.word Start
	.word Start