; Player/Missile demo #2
; Multiple enemies
; Charles Wieland, 6/25/2014
; junkosoft.com

	processor 6502
	include vcs.h

; Constant values
MINHORZPOS = $18
MAXHORZPOS = $9E
CENTERPOS = $5B
NUMPLAYAREALINES = $90
LINESFORENEMYROW = $10
NUMENEMYROWS = $07

; Labels for RAM
; Horizontal position of the enemy sprite
; $80 - $89
ENEMYHORZPOS = $80

; Row of last enemy that was hit
ENEMYHIT = $8A

; Offset between current scan line and the render location of the enemy sprite
ENEMYVERTPOSTEMP = $8B

; Current enemy row
ENEMYROW = $8C

; Horizontal position of the player sprite
PLAYERHORZPOS = $8D

; Vertical position of the player's missile
MISSILEVERTPOS = $8E

; Offset between current scan line and the render location of the player's missile
MISSILEVERTPOSTEMP = $8F

; Horizontal position of the player's missile
MISSILEHORZPOS = $90

; $00 when the button is released, $01 when the button is being held
BUTTONDOWN = $91

; Shooting sound effect counter
SHOOTSOUNDCOUNTER = $92

; Score, number of times enemy hit
SCORE = $93

; Number of frames when enemy is hit
DEATHFRAMECOUNT = $95

; Incremented for each frame, may be used for random number generation later??
FRAMECOUNT = $96

	ORG $F000
Start	
; Initialize the system
	SEI
	CLD
	LDX #$FF
	TXS
	LDA #$00

ClearStack
	STA $00,X
	DEX
	BNE ClearStack

; Define initial values
	LDA #CENTERPOS
	STA PLAYERHORZPOS
	LDA #$2E
	STA COLUP0
	LDA #$01
	STA CTRLPF
	LDA #$0F
	STA AUDC0
	LDA #$08
	STA AUDC1
	LDX #NUMENEMYROWS
	LDA #MINHORZPOS
	
ProcessNextHorizontalValue
	STA ENEMYHORZPOS,X
	DEX
	BPL ProcessNextHorizontalValue
	
; Start vertical blank	
DrawVBlank
	LDA #$02
	STA VSYNC
	STA WSYNC
	STA WSYNC
	STA WSYNC
	LDA #$00
	STA VSYNC
	STA VBLANK

; Set the timer
	LDA #$2C
	STA TIM64T

; Increment frame count and get the current ship color
	INC FRAMECOUNT
	LDA DEATHFRAMECOUNT
	STA AUDV1
	BEQ HandleCollisions
	
	STA COLUP1
	STA AUDF1
	LDX ENEMYHIT
	DEC DEATHFRAMECOUNT
	BEQ ResetEnemyLeft
	JMP CheckJoystick	

HandleCollisions
; Handle collisions
	LDA CXM0P
	AND #$80
	BEQ MoveEnemies
	
; Number of lines minus four to adjust for size of missile
	LDA #NUMPLAYAREALINES-8
	SBC MISSILEVERTPOS
	AND #$F0
	LSR
	LSR
	LSR
	LSR
	STA ENEMYHIT
	TAX
	INX
	TXA
	ADC SCORE
	BCS IncrementHighByte
	JMP SaveScore

IncrementHighByte
	INC SCORE+1

SaveScore	
	STA SCORE
	LDA #$00
	STA MISSILEVERTPOS
	
	LDA #$0F
	STA DEATHFRAMECOUNT

MoveEnemies
	LDX #NUMENEMYROWS

ProcessRow
	TXA
	AND #$01
	BEQ MoveEnemyLeft
	
	; Move Enemy Right
	LDA #MAXHORZPOS
	CMP ENEMYHORZPOS,X
	BEQ ResetEnemyRight

	INC ENEMYHORZPOS,X
	JMP ProcessNextEnemy
	
MoveEnemyLeft
	LDA #MINHORZPOS
	CMP ENEMYHORZPOS,X
	BEQ ResetEnemyLeft
	
	DEC ENEMYHORZPOS,X
	JMP ProcessNextEnemy

ResetEnemyLeft
	LDA #MAXHORZPOS
	STA ENEMYHORZPOS,X
	JMP ProcessNextEnemy

ResetEnemyRight
	LDA #MINHORZPOS
	STA ENEMYHORZPOS,X

ProcessNextEnemy
	DEX
	BPL ProcessRow
	
SetEnemyColors
	LDA SCORE
	AND #$70
	LSR
	LSR
	LSR
	LSR
	TAX
	LDA ShipColors,X
	STA COLUP1

CheckJoystick
	LDA #$40
	BIT SWCHA		; Left
	BEQ MovePlayerLeft
	
	LDA #$80
	BIT SWCHA		; Right
	BEQ MovePlayerRight
	JMP CheckButton
	
MovePlayerLeft
	LDA PLAYERHORZPOS
	CMP #MINHORZPOS
	BEQ CheckButton
	
	DEC PLAYERHORZPOS
	JMP CheckButton

MovePlayerRight
	LDA PLAYERHORZPOS
	CMP #MAXHORZPOS
	BEQ CheckButton

	INC PLAYERHORZPOS

CheckButton
	STA CXCLR
	LDA INPT4
    BMI ClearButtonFlag
    
	LDA BUTTONDOWN
	BNE PositionObjects	

SetButtonFlag
; Flag button as pressed
	LDA #$01
	STA BUTTONDOWN
	
; Make sure a missile isn't launched
	LDA MISSILEVERTPOS
	BNE PositionObjects
	
; Copy current player horizontal position into missile horizontal position, adjusting to center missile over player
	LDA PLAYERHORZPOS
	ADC #$04
	STA MISSILEHORZPOS
	
; Set missile's vertical position slightly above player
	LDA #NUMPLAYAREALINES
	STA MISSILEVERTPOS
	
; Set sound effect counter
	LDA #$E0
	STA SHOOTSOUNDCOUNTER

ClearButtonFlag
; Button released
	LDA #$00
	STA BUTTONDOWN
	
PositionObjects	
; Position the missile
	LDX #$02
	LDA MISSILEHORZPOS
	JSR PositionObject
	STA WSYNC

; Position the player
	LDX #$00
	LDA PLAYERHORZPOS
	JSR PositionObject
	STA WSYNC
	
; Do the sound effect
	LDA SHOOTSOUNDCOUNTER
	BEQ DisableSound
	
	INC SHOOTSOUNDCOUNTER
	STA AUDF0
	LDA #$0F
	
DisableSound
	STA AUDV0
	
; Copy vertical positions into temp locations
ResetValues
	LDA #LINESFORENEMYROW
	STA ENEMYVERTPOSTEMP
	LDA #NUMENEMYROWS
	STA ENEMYROW
	LDA #$00
	LDA MISSILEVERTPOS
	STA MISSILEVERTPOSTEMP
	
WaitForTimer
	LDA INTIM
	BNE WaitForTimer
	STA WSYNC
    STA VBLANK

; Define the playfield
	LDA #$98
	STA COLUBK	
	LDY #$20

;  Draws 32 lines of score
;  TODO: implement!
DrawScoreArea
	STY COLUPF
	STA WSYNC
	DEY
	BNE DrawScoreArea
	
	LDA #$CE
	STA COLUPF
	
	LDA #$00						; 2
	STA PF0							; 3
	STA PF1							; 2
	STA PF2							; 2

; Position the first enemy
	LDX #NUMENEMYROWS
	LDA ENEMYHORZPOS,X
	LDX #$01
	JSR PositionObject

; Draw 144 lines of the play area (minus to two to adjust for horizontal position above)
	LDY #NUMPLAYAREALINES-2        	; 2
	
DrawPlayArea
	LDA #$F8						; 2
	DEC MISSILEVERTPOSTEMP			; 5
	DEC ENEMYVERTPOSTEMP			; 5
	STA WSYNC						; 3

	BIT ENEMYVERTPOSTEMP			; 3
	BNE CheckMissilePosition		; 2

	LDX ENEMYVERTPOSTEMP			; 3
	LDA Ship,X						; 3
	STA GRP1						; 3
	
	CPX #$00						; 2
	BNE CheckMissilePosition		; 2

; Position the next enemy
	DEC ENEMYROW
	BMI CheckMissilePosition

	LDX ENEMYROW
	LDA ENEMYHORZPOS,X
	LDX #$01
	JSR PositionObject
	DEY
	DEY

	LDA #LINESFORENEMYROW			; 2
	STA ENEMYVERTPOSTEMP			; 3
	
CheckMissilePosition
	LDA #$FC						; 2
	BIT MISSILEVERTPOSTEMP			; 3
	BNE FinishScanline				; 2
	
	LDA #$02						; 2

FinishScanline
	STA ENAM0						; 3
	DEY								; 2
	BNE DrawPlayArea				; 2
	
	; Draw player, 8 lines
	LDX #$07						; 2
	LDA #$30						; 2
	STA PF0							; 3
	
DrawPlayer
	STA WSYNC
	LDA P0Data,X
	STA GRP0
	DEX
	BPL DrawPlayer

;  Draws 8 lines of blue border
	LDY #$CE
	LDA #$FF
	STA PF0
	STA PF1
	STA PF2
	INX
	STX GRP0
	STX ENAM0

DrawBottom
	STY COLUPF
	STA WSYNC
	DEY
	DEY
	CPY #$BE
	BNE DrawBottom
	
	; Clear out TIA registers
	LDA #$00
	STA PF0
	STA PF1
	STA PF2
	STA COLUBK
	
; Check position of missile
	LDA MISSILEVERTPOS
	BEQ DrawOverscan
	
; Change vertical position of missile
	DEC MISSILEVERTPOS
	DEC MISSILEVERTPOS
	LDA #$02
	STA VBLANK
	
; DrawOverscan (30 scanlines)
DrawOverscan
	LDY #$1E
DrawOverscanline
    STA WSYNC
	DEY
	BNE DrawOverscanline
	JMP DrawVBlank

PositionObject:
	STA HMCLR
	SEC
	STA WSYNC

WaitForPosition:
	SBC #$0F
	BCS WaitForPosition

	EOR #$07
	ASL
	ASL
	ASL
	ASL
	STA RESP0,X
	STA HMP0,X
	STA WSYNC
	STA HMOVE
	RTS
	
Ship
	.byte %00000000
	.byte %00011000
	.byte %00011000
	.byte %11111111
	.byte %00000000
	.byte %11111111
	.byte %00011000
	.byte %00011000
	
ShipColors
	.byte $0E
	.byte $1E
	.byte $3E
	.byte $42
	.byte $60
	.byte $B4
	.byte $CE
	.byte $FE
	
P0Data
	.byte %11100111
	.byte %11100111
	.byte %11100111
	.byte %01111110
	.byte %00111100
	.byte %00111100
	.byte %00011000
	.byte %00011000
	
	org $FFEB
	dc.b "(c)2014 JunkoSoft"
	.word Start
	.word Start