; Player/Missile demo #4
; Multiple enemies with six digit score and enemy missile
; Charles Wieland, 6/27/2014
; junkosoft.com

	processor 6502
	include vcs.h

; Constant values
MINHORZPOS = $18
MAXHORZPOS = $9E
CENTERPOS = $5B
NUMPLAYAREALINES = $9A
LINESFORENEMYROW = $10
NUMENEMYROWS = $07 ; 8 rows - $00 to $07

; Labels for RAM
; Horizontal position of the enemy sprite
; $80 - $87
ENEMYHORZPOS = $80

; Row of last enemy that was hit
ENEMYHIT = $8A

; Offset between current scan line and the render location of the enemy sprite
ENEMYVERTPOSTEMP = $8B

; Current enemy row
ENEMYROW = $8C

; Horizontal position of the player sprite
PLAYERHORZPOS = $8D

; Vertical position of the player's missile
PLAYERMISSILEVERTPOS = $8E

; Offset between current scan line and the render location of the player's missile
PLAYERMISSILEVERTPOSTEMP = $8F

; Horizontal position of the player's missile
PLAYERMISSILEHORZPOS = $90

; $00 when the button is released, $01 when the button is being held
BUTTONDOWN = $91

; Shooting sound effect counter
SHOOTSOUNDCOUNTER = $92

; Number of frames when enemy is hit
DEATHFRAMECOUNT = $93

; Color of the score
SCORECOLOR = $94

; Score, number of times enemy hit
; 2 bytes, acts like two base ten digits per byte
; $95 - 100,000 and 10,000 digits
SCORE1 = $95
; $96 - 1000 and 100 digits
SCORE2 = $96
; $97 - 10 and 1 digits
SCORE3 = $97

; Word value that points to current first digit
DIGITPOINTER1 = $98

; Word value that points to current second digit
DIGITPOINTER2 = $9A

; Word value that points to current third digit
DIGITPOINTER3 = $9C

; Word value that points to current fourth digit
DIGITPOINTER4 = $9E

; Word value that points to current fourth digit
DIGITPOINTER5 = $A0

; Word value that points to current fourth digit
DIGITPOINTER6 = $A2

; Current scanline when drawing the score
SCORESCANLINE = $A4

; Temporary value used when drawing score
TEMPDIGITVALUE = $A5

; Color of enemy
; 8 bytes $A6 - $AD
ENEMYCOLOR = $A6

; Vertical position of the enemy missile
ENEMYPLAYERMISSILEVERTPOS = $AE

; Horizontal position of the enemy missile
ENEMYPLAYERMISSILEHORZPOS = $AF

; Offset between current scan line and the render location of the enemy's missile
ENEMYPLAYERMISSILEVERTPOSTEMP = $B0

; Current background color
BACKGROUNDCOLOR = $B1

; 
PLAYERDEATHFRAMECOUNT = $B2

; Incremented for each frame, may be used for random number generation later??
FRAMECOUNT = $C0

	ORG $F000
Start	
; Initialize the system
	SEI
	CLD
	LDX #$FF
	TXS
	LDA #$00

ClearStack
	STA $00,X
	DEX
	BNE ClearStack

; Define initial values
	LDA #CENTERPOS
	STA PLAYERHORZPOS
	LDA #$01
	STA CTRLPF
	LDA #$0F
	STA SCORECOLOR
	LDA #$08
	STA AUDC1
		
; Initialize all of the pointers to $FFFF
	LDX #$0C
	LDA #$FF
SetPointerValue
	STA DIGITPOINTER1,X
	DEX
	BPL SetPointerValue

; Set the initial enemy positions
	LDX #NUMENEMYROWS
	LDA #MINHORZPOS	
ProcessNextHorizontalValue
	STA ENEMYHORZPOS,X
	DEX
	BPL ProcessNextHorizontalValue
	
; Start vertical blank	
DrawVBlank
	LDA #$02
	STA VSYNC
	STA WSYNC
	STA WSYNC
	STA WSYNC
	LDA #$00
	STA VSYNC
	STA VBLANK

; Set the timer
	LDA #$2C
	STA TIM64T

; Increment frame count
	INC FRAMECOUNT

; See if the player has been hit
	LDA PLAYERDEATHFRAMECOUNT
	BEQ CheckEmenyDead

	LDA PLAYERDEATHFRAMECOUNT
	STA BACKGROUNDCOLOR
	STA AUDF0
	LDA #$0F
	STA AUDC0
	LDA #$0C
	STA AUDV0

	DEC PLAYERDEATHFRAMECOUNT
	JMP ResetValues
	
CheckEmenyDead	
	LDA DEATHFRAMECOUNT
	STA AUDV1
	BEQ CheckEnemyCollision
	
	LDX ENEMYHIT
	STA ENEMYCOLOR,X
	STA AUDF1
	DEC DEATHFRAMECOUNT
	BEQ ResetDeadEnemy
	JMP CheckPlayerCollision

; Properly reset killed enemy
ResetDeadEnemy
	TXA
	AND #$01
	BEQ ResetEnemyLeft
	BNE ResetEnemyRight
	
CheckEnemyCollision
; Handle collisions
	LDA CXM0P
	AND #$80
	BEQ CheckPlayerCollision
	
; Get the row of the current enemy that was hit
	LDA #NUMPLAYAREALINES-24
	SBC PLAYERMISSILEVERTPOS
	AND #$F0
	LSR
	LSR
	LSR
	LSR
	STA ENEMYHIT
	TAX

; Increment row of enemy hit and store it in the temp variable
	INX
	STX TEMPDIGITVALUE

; Turn on decimal mode and add the score of the enemy that was hit
	SED
	CLC
	LDA SCORE3
	ADC TEMPDIGITVALUE
	STA SCORE3
	LDA SCORE2
	ADC #$00
	STA SCORE2
	LDA SCORE1
	ADC #$00
	STA SCORE1
	CLD

	LDA #$00
	STA PLAYERMISSILEVERTPOS
	
	LDA #$0F
	STA DEATHFRAMECOUNT
	
CheckPlayerCollision
; Handle collisions
	LDA CXM1P
	AND #$80
	BEQ SetBackgroundColor

	LDA #$00
	STA PLAYERMISSILEVERTPOS
	STA ENEMYPLAYERMISSILEVERTPOS
	
; Set the death frame count
	LDA #$0F
	STA PLAYERDEATHFRAMECOUNT
	
SetBackgroundColor
	LDA #$98
	STA BACKGROUNDCOLOR
	
MoveEnemies
	LDX #NUMENEMYROWS

ProcessRow
	CPX ENEMYHIT
	BNE MoveEnemy

	LDA DEATHFRAMECOUNT
	BNE ProcessNextEnemy

MoveEnemy
	TXA
	AND #$01
	BEQ MoveEnemyLeft
	
	; Move Enemy Right
	LDA #MAXHORZPOS
	CMP ENEMYHORZPOS,X
	BEQ ResetEnemyRight

	INC ENEMYHORZPOS,X
	JMP SetEnemyColor
	
MoveEnemyLeft
	LDA #MINHORZPOS
	CMP ENEMYHORZPOS,X
	BEQ ResetEnemyLeft
	
	DEC ENEMYHORZPOS,X
	JMP SetEnemyColor

ResetEnemyLeft
	LDA #MAXHORZPOS
	STA ENEMYHORZPOS,X
	JMP SetEnemyColor

ResetEnemyRight
	LDA #MINHORZPOS
	STA ENEMYHORZPOS,X

SetEnemyColor
	LDA SCORE2
	AND #$F0
	LSR
	LSR
	LSR
	LSR
	TAY
	LDA ShipColors,Y
	STA ENEMYCOLOR,X
	
ProcessNextEnemy
	DEX
	BPL ProcessRow
	
CheckJoystick
	LDA #$40
	BIT SWCHA		; Left
	BEQ MovePlayerLeft
	
	LDA #$80
	BIT SWCHA		; Right
	BEQ MovePlayerRight
	JMP CheckButton
	
MovePlayerLeft
	LDA PLAYERHORZPOS
	CMP #MINHORZPOS
	BEQ CheckButton
	
	DEC PLAYERHORZPOS
	JMP CheckButton

MovePlayerRight
	LDA PLAYERHORZPOS
	CMP #MAXHORZPOS
	BEQ CheckButton

	INC PLAYERHORZPOS

CheckButton
	STA CXCLR
	LDA INPT4
    BMI ClearButtonFlag
    
	LDA BUTTONDOWN
	BNE CheckEnemyMissile	

SetButtonFlag
; Flag button as pressed
	LDA #$01
	STA BUTTONDOWN
	
; Make sure a missile isn't launched
	LDA PLAYERMISSILEVERTPOS
	BNE CheckEnemyMissile
	
; Copy current player horizontal position into missile horizontal position, adjusting to center missile over player
	LDA PLAYERHORZPOS
	ADC #$04
	STA PLAYERMISSILEHORZPOS
	
; Set missile's vertical position slightly above player
	LDA #NUMPLAYAREALINES-16
	STA PLAYERMISSILEVERTPOS
	
; Set sound effect counter
	LDA #$E0
	STA SHOOTSOUNDCOUNTER

ClearButtonFlag
; Button released
	LDA #$00
	STA BUTTONDOWN

CheckEnemyMissile
	LDA ENEMYPLAYERMISSILEVERTPOS
	BEQ LaunchEnemyMissile
	
	CMP #NUMPLAYAREALINES
	BNE MoveEnemyMissile
	
; Disable enemy missile
	LDA #$00
	STA ENEMYPLAYERMISSILEVERTPOS
	BEQ SetPlayerMissilePos
	
MoveEnemyMissile
	INC ENEMYPLAYERMISSILEVERTPOS
	JMP SetPlayerMissilePos
	
LaunchEnemyMissile
	LDX #$00

CheckEnemyPos
	LDA ENEMYHORZPOS,X
	ADC #04
	CMP PLAYERHORZPOS
	BEQ SetEnemyMissilePos
	
	INX
	CPX #$08
	BEQ SetPlayerMissilePos
	BNE CheckEnemyPos
	
SetEnemyMissilePos
	ADC #$04
	STA ENEMYPLAYERMISSILEHORZPOS
	LDA EnemyVertPos,X
	STA ENEMYPLAYERMISSILEVERTPOS

SetPlayerMissilePos
; Check position of missile
	LDA PLAYERMISSILEVERTPOS
	BEQ DoShotSoundEffect
	
; Change vertical position of missile
	DEC PLAYERMISSILEVERTPOS
	DEC PLAYERMISSILEVERTPOS

DoShotSoundEffect	
; Do the sound effect
	LDA SHOOTSOUNDCOUNTER
	BEQ DisableSound
	
	INC SHOOTSOUNDCOUNTER
	STA AUDF0
	LDA #$0F
	STA AUDC0
	
DisableSound
	STA AUDV0
	
; Copy vertical positions into temp locations
ResetValues
	LDA #LINESFORENEMYROW
	STA ENEMYVERTPOSTEMP
	LDA #NUMENEMYROWS
	STA ENEMYROW
	LDA #$00
	LDA PLAYERMISSILEVERTPOS
	STA PLAYERMISSILEVERTPOSTEMP
	LDA ENEMYPLAYERMISSILEVERTPOS
	STA ENEMYPLAYERMISSILEVERTPOSTEMP

; Set digit pointers
	LDA SCORE3
	JSR CalculateScoreDigitPointer
	STY DIGITPOINTER6
	STX DIGITPOINTER5
	LDA SCORE2
	JSR CalculateScoreDigitPointer
	STY DIGITPOINTER4
	STX DIGITPOINTER3
	LDA SCORE1
	JSR CalculateScoreDigitPointer
	STY DIGITPOINTER2
	STX DIGITPOINTER1
	
WaitForTimer
	LDA INTIM
	BNE WaitForTimer
	STA WSYNC
    STA VBLANK

; Set the background color
	LDA BACKGROUNDCOLOR
	STA COLUBK	
	LDY #$20

; Draw the score
DrawScore
	STA WSYNC
	LDA #$00
	STA GRP0
	STA GRP1    
	STA WSYNC
	NOP
	STA GRP0 
	STA GRP1 
	LDY #$07 
	STY SCORESCANLINE  
	LDA #$03 
	STA NUSIZ0  
	STA NUSIZ1  
	STA VDELP0  
	STA VDELP1  
	LDA #$F0 
	STA HMP0 
	LDA SCORECOLOR
	NOP
	STA RESP0
	STA RESP1
	STA COLUP0  
	STA COLUP1  
	STA WSYNC
	STA HMOVE
	
DrawScoreScanline
	LDY SCORESCANLINE  
	LDA (DIGITPOINTER1),Y 
	STA GRP0 
	STA WSYNC
	LDA (DIGITPOINTER2),Y 
	STA GRP1 
	LDA (DIGITPOINTER3),Y 
	STA GRP0 
	LDA (DIGITPOINTER4),Y 
	STA TEMPDIGITVALUE  
	LDA (DIGITPOINTER5),Y 
	TAX
	LDA (DIGITPOINTER6),Y 
	TAY
	LDA TEMPDIGITVALUE  
	STA GRP1 
	STX GRP0 
	STY GRP1 
	STA GRP0 
	DEC SCORESCANLINE  
	BPL DrawScoreScanline
	STA WSYNC

; Clear out players after drawing score
	LDA #$00 
	STA GRP0 
	STA GRP1 
	STA VDELP0  
	STA VDELP1  
	STA NUSIZ0  
	STA NUSIZ1
	LDA #$2E
	STA COLUP0
	LDA #$CE						; 2
	STA COLUPF						; 3
	LDX #NUMENEMYROWS
	STA WSYNC

; Position the first enemy
	LDA ENEMYHORZPOS,X
	LDX #$01
	JSR PositionObject
	STA WSYNC

; Position the enemy's missile	
	LDX #$03
	LDA ENEMYPLAYERMISSILEHORZPOS
	JSR PositionObject
	STA WSYNC
	
; Position the player's missile
	LDX #$02
	LDA PLAYERMISSILEHORZPOS
	JSR PositionObject
	
	LDX #$07
	LDA ENEMYCOLOR,X
	STA COLUP1

; Draw 144 lines of the play area (minus to two to adjust for horizontal position above)
	LDY #NUMPLAYAREALINES-2        	; 2
	
DrawPlayArea
	LDA #$F8						; 2
	DEC PLAYERMISSILEVERTPOSTEMP			; 5
	DEC ENEMYVERTPOSTEMP			; 5
	DEC ENEMYPLAYERMISSILEVERTPOSTEMP		; 5
	STA WSYNC						; 3

	BIT ENEMYVERTPOSTEMP			; 3
	BNE CheckMissilePosition		; 2

	LDX ENEMYVERTPOSTEMP			; 3
	LDA Ship,X						; 3
	STA GRP1						; 3
	
	CPX #$00						; 2
	BNE CheckMissilePosition		; 2

; Position the next enemy and get the ship color
	DEC ENEMYROW
	LDX ENEMYROW
	LDA ENEMYCOLOR,X
	STA COLUP1
	CPX #$00
	BMI CheckMissilePosition

	LDX ENEMYROW
	LDA ENEMYHORZPOS,X
	LDX #$01
	JSR PositionObject
	DEY
	DEY

	LDA #LINESFORENEMYROW			; 2
	STA ENEMYVERTPOSTEMP			; 3
	
CheckMissilePosition
	LDA #$FC						; 2
	BIT PLAYERMISSILEVERTPOSTEMP			; 3
	BNE CheckEnemyMissilePosition	; 2
	
	LDA #$02						; 2

CheckEnemyMissilePosition
	STA ENAM0						; 3

	LDA #$FC						; 2
	BIT ENEMYPLAYERMISSILEVERTPOSTEMP		; 3
	BNE FinishScanline				; 2
	
	LDA #$02						; 2
	
FinishScanline
	STA ENAM1						; 2
	DEY								; 2
	BNE DrawPlayArea				; 2
	
; Position the player
	LDX #$00
	LDA PLAYERHORZPOS
	JSR PositionObject
	
	LDA #$30						; 2
	STA PF0							; 3

	; Draw player, 8 lines
	LDX #$07						; 2
	
DrawPlayer
	STA WSYNC
	LDA P0Data,X
	STA GRP0
	DEX
	BPL DrawPlayer

;  Draws 8 lines of green border
	LDY #$CE
	LDA #$FF
	STA PF0
	STA PF1
	STA PF2
	INX
	STX GRP0
	STX ENAM0
	STX ENAM1
	
DrawBottom
	STY COLUPF
	STA WSYNC
	DEY
	DEY
	CPY #$BE
	BNE DrawBottom
	
	; Clear out TIA registers
	LDA #$00
	STA PF0
	STA PF1
	STA PF2
	STA COLUBK
	
	LDA #$02
	STA VBLANK
	
; DrawOverscan (30 scanlines)
DrawOverscan
	LDY #$1E
DrawOverscanline
    STA WSYNC
	DEY
	BNE DrawOverscanline
	JMP DrawVBlank

PositionObject:
	STA HMCLR
	SEC
	STA WSYNC

WaitForPosition:
	SBC #$0F
	BCS WaitForPosition

	EOR #$07
	ASL
	ASL
	ASL
	ASL
	STA RESP0,X
	STA HMP0,X
	STA WSYNC
	STA HMOVE
	RTS
	
; Gets the low byte of the pointer for each digit
CalculateScoreDigitPointer:
	TAX
	AND #$0F 
	ASL
	ASL
	ASL
	TAY
	TXA
	AND #$F0 
	LSR
	TAX
	RTS

Ship
	.byte %00000000
	.byte %00011000
	.byte %00011000
	.byte %11111111
	.byte %00000000
	.byte %11111111
	.byte %00011000
	.byte %00011000
	
ShipColors
	.byte $0E ; White
	.byte $1E ; Yellow
	.byte $3E ; Orange
	.byte $44 ; Dark Red
	.byte $50 ; Maroon
	.byte $60 ; Purple
	.byte $B2 ; Green
	.byte $CE ; Light Green
	.byte $FE ; Tan
	.byte $00 ; Black

EnemyVertPos
	.byte $89,$77,$65,$53,$41,$2F,$1D,$0B

P0Data
	.byte %11100111 ; |###  ###|
	.byte %11100111 ; |###  ###|
	.byte %11100111 ; |###  ###|
	.byte %01111110 ; | ###### |
	.byte %00111100 ; |  ####  |
	.byte %00111100 ; |  ####  |
	.byte %00011000 ; |   ##   |
	.byte %00011000 ; |   ##   |
	
	ORG $FF00
	; Zero
	.byte $3C ; |  ####  |
	.byte $66 ; | ##  ## |
	.byte $66 ; | ##  ## |
	.byte $66 ; | ##  ## |
	.byte $66 ; | ##  ## |
	.byte $66 ; | ##  ## |
	.byte $66 ; | ##  ## |
	.byte $3C ; |  ####  |

	; One
	.byte $7E ; | ###### |
	.byte $18 ; |   ##   |
	.byte $18 ; |   ##   |
	.byte $18 ; |   ##   |
	.byte $18 ; |   ##   |
	.byte $38 ; |  ###   |
	.byte $18 ; |   ##   |
	.byte $08 ; |    #   |

	; Two
	.byte $7E ; | ###### |
	.byte $60 ; | ##     |
	.byte $60 ; | ##     |
	.byte $3C ; |  ####  |
	.byte $06 ; |     ## |
	.byte $06 ; |     ## |
	.byte $46 ; | #   ## |
	.byte $3C ; |  ####  |

	; Three
	.byte $3C ; |  ####  |
	.byte $46 ; | #   ## |
	.byte $06 ; |     ## |
	.byte $06 ; |     ## |
	.byte $1C ; |   ###  |
	.byte $06 ; |     ## |
	.byte $46 ; | #   ## |
	.byte $3C ; |  ####  |

	; Four
	.byte $0C ; |    ##  |
	.byte $0C ; |    ##  |
	.byte $7E ; | ###### |
	.byte $4C ; | #  ##  |
	.byte $4C ; | #  ##  |
	.byte $2C ; |  # ##  |
	.byte $1C ; |   ###  |
	.byte $0C ; |    ##  |

	; Five
	.byte $3C ; |  ####  |
	.byte $46 ; | #   ## |
	.byte $06 ; |     ## |
	.byte $06 ; |     ## |
	.byte $3C ; |  ####  |
	.byte $60 ; | ##     |
	.byte $60 ; | ##     |
	.byte $7E ; | ###### |

	; Six
	.byte $3C ; |  ####  |
	.byte $66 ; | ##  ## |
	.byte $66 ; | ##  ## |
	.byte $66 ; | ##  ## |
	.byte $7C ; | #####  |
	.byte $60 ; | ##     |
	.byte $62 ; | ##   # |
	.byte $3C ; |  ####  |

	; Seven
	.byte $30 ; |  ##    |
	.byte $30 ; |  ##    |
	.byte $30 ; |  ##    |
	.byte $18 ; |   ##   |
	.byte $0C ; |    ##  |
	.byte $06 ; |     ## |
	.byte $42 ; | #    # |
	.byte $3E ; |  ##### |

	; Eight
	.byte $3C ; |  ####  |
	.byte $66 ; | ##  ## |
	.byte $66 ; | ##  ## |
	.byte $66 ; | ##  ## |
	.byte $3C ; |  ####  |
	.byte $66 ; | ##  ## |
	.byte $66 ; | ##  ## |
	.byte $3C ; |  ####  |

	; Nine
	.byte $3C ; |  ####  |
	.byte $46 ; | #   ## |
	.byte $06 ; |     ## |
	.byte $3E ; |  ##### |
	.byte $66 ; | ##  ## |
	.byte $66 ; | ##  ## |
	.byte $66 ; | ##  ## |
	.byte $3C ; |  ####  |
	
	org $FFEB
	dc.b "(c)2014 JunkoSoft"
	.word Start
	.word Start