; Player/Missile demo #6
; Multiple enemies with six digit score and enemy missile
; Charles Wieland, 6/8/2015
; junkosoft.com

; Changes:
;   Removal of enemy sprites when shot

; Bugs:
;   Top enemy cannot shoot

; Future changes:
;   Incorporation of NUSIZ0 values
;   Color scrolling when level not started
;   Select switch and modes
;   Updated logic for enemy missiles (multiple?)
;   Add junkosoft logo
;   Vertical and random enemy movements
;   Levels with timer
;   Speed increases when level changes
;   Extra men at 1000 point intervals
;   Lost lives bug (multiple lost at once)

	processor 6502
	include vcs.h

; Constant values
MINHORZPOS = $18
MAXHORZPOS = $9E
CENTERPOS = $5B
NUMPLAYAREALINES = $9A
LINESFORENEMYROW = $10
NUMENEMYROWS = $07 ; 8 rows - $00 to $07
VSYNC_TIME = $28
VBLANK_TIME = $2C
OSCAN_TIME = $22

; Labels for RAM
	SEG.U VARS
	ORG $80
	
; Horizontal position of the enemy sprite row
ENEMYHORZPOS ds 8

; Row of last enemy that was hit
ENEMYHIT ds 1

; Offset between current scan line and the render location of the enemy sprite
ENEMYVERTPOSTEMP ds 1

; Current enemy row
ENEMYROW ds 1

; Horizontal position of the player sprite
PLAYERHORZPOS ds 1

; Vertical position of the player's missile
PLAYERMISSILEVERTPOS ds 1

; Offset between current scan line and the render location of the player's missile
PLAYERMISSILEVERTPOSTEMP ds 1

; Horizontal position of the player's missile
PLAYERMISSILEHORZPOS ds 1

; $00 when the button is released, $01 when the button is being held
BUTTONDOWN ds 1

; Shooting sound effect counter
SHOOTSOUNDCOUNTER ds 1

; Number of frames when enemy is hit
DEATHFRAMECOUNT ds 1

; Color of the score
SCORECOLOR ds 1

; Score, number of times enemy hit
; 2 bytes, acts like two base ten digits per byte
; $95 - 100,000 and 10,000 digits
SCORE1 ds 1
; $96 - 1000 and 100 digits
SCORE2 ds 1
; $97 - 10 and 1 digits
SCORE3 ds 1

; Word value that points to current first digit
DIGITPOINTER1 ds 2

; Word value that points to current second digit
DIGITPOINTER2 ds 2

; Word value that points to current third digit
DIGITPOINTER3 ds 2

; Word value that points to current fourth digit
DIGITPOINTER4 ds 2

; Word value that points to current fourth digit
DIGITPOINTER5 ds 2

; Word value that points to current fourth digit
DIGITPOINTER6 ds 2

; Current scanline when drawing the score
SCORESCANLINE ds 1

; Temporary value used when drawing score
TEMPDIGITVALUE ds 1

; Color of enemy
; 8 bytes $A6 - $AD
ENEMYCOLOR ds 8

; Vertical position of the enemy missile
ENEMYPLAYERMISSILEVERTPOS ds 1

; Horizontal position of the enemy missile
ENEMYPLAYERMISSILEHORZPOS ds 1

; Offset between current scan line and the render location of the enemy's missile
ENEMYPLAYERMISSILEVERTPOSTEMP ds 1

; Current background color
BACKGROUNDCOLOR ds 1

; Frame count used when player sprite has been hit
PLAYERDEATHFRAMECOUNT ds 1

; Current state of the game
; 0 - active game
; 1 - not yet started or player dead
; 2 - reset button is pressed down or player out of lives
GAMESTATE ds 1

; Random number
RAND ds 1

; Number of player lives
PLAYERLIVES ds 1

; Graphics Mask for the current row
MASK ds 8

; Temporary storage
TEMP ds 1

; Incremented for each frame, may be used for random number generation later??
FRAMECOUNT ds 1

; Declare program section
    SEG CODE  	  
	ORG $F000

PositionObject:
	STA HMCLR
	SEC
	STA WSYNC

WaitForPosition:
	SBC #$0F
	BCS WaitForPosition

	EOR #$07
	ASL
	ASL
	ASL
	ASL
	STA RESP0,X
	STA HMP0,X
	STA WSYNC
	STA HMOVE
	RTS

Start	
; Initialize the system
	SEI
	CLD
	LDX #$FF
	TXS
	LDA #$00

ClearStack
	STA $00,X
	DEX
	BNE ClearStack

; Define initial values
	LDA #$01
	STA CTRLPF
	LDA #$08
	STA AUDC1
	JSR SetInitialValues
		
; Initialize all of the pointers to $FFFF
	LDX #$0C
	LDA #$FF
SetPointerValue
	STA DIGITPOINTER1,X
	DEX
	BPL SetPointerValue
	JSR PositionEnemies	
	
DrawVerticalSync
	LDA #$FF
	STA VSYNC
	STA VBLANK
	
	LDA #VSYNC_TIME
	STA TIM8T
	
; Increment frame count
	INC FRAMECOUNT
	
WaitForVerticalSync
	LDA INTIM
	BNE WaitForVerticalSync
	STA WSYNC
	STA VSYNC
	
; Set the timer for VBlank
DrawVBlank	
	LDA #VBLANK_TIME
	STA TIM64T
	
	LDA SWCHB
	LSR
	BCC ResetPressed
	
; If reset button was just released, setup new game
	LDA GAMESTATE
	AND #$02
	BNE ResetReleased
	JMP GameStarted
	
ResetPressed
	; Set the game status to "reset is pressed down"
	LDA #$02
	STA GAMESTATE
	
	; Remove missiles
	LDA #$00
	STA PLAYERMISSILEVERTPOS
	STA ENEMYPLAYERMISSILEVERTPOS
	STA AUDV0
	STA AUDV1	
	JMP ResetValues

ResetReleased
	; Reset the values
	JSR SetInitialValues
	
	; Reset initial position of the enemies
	LDA FRAMECOUNT
	STA RAND
	JSR PositionEnemies
	
GameStarted
; See if the player has been hit
	LDA PLAYERDEATHFRAMECOUNT
	BEQ CheckPlayerOutOfLives

	LDA PLAYERDEATHFRAMECOUNT
	STA BACKGROUNDCOLOR
	STA AUDF0
	LDA #$0F
	STA AUDC0
	LDA #$0C
	STA AUDV0

	DEC PLAYERDEATHFRAMECOUNT
	JMP ResetValues

; Stop the game if player is out of lives
CheckPlayerOutOfLives
	LDA GAMESTATE
	AND #$04
	BEQ CheckEnemyDead
	
	LDA #$00
	STA AUDV0
	STA AUDV1	
	JMP ResetValues
	
CheckEnemyDead	
	LDA DEATHFRAMECOUNT
	STA AUDV1
	BEQ CheckEnemyCollision
	
	LDX ENEMYHIT
	STA ENEMYCOLOR,X
	STA AUDF1
	DEC DEATHFRAMECOUNT
	BEQ ResetDeadEnemy
	JMP CheckPlayerCollision

; Properly reset killed enemy
ResetDeadEnemy
; Clear mask for the row
	LDA #$00
	STA MASK,X
	STX TEMP
	LDX #$07
	
CheckRowMask
	LDA MASK,X
	BNE ResetDeadEnemyPosition
	DEX
	BPL CheckRowMask
	
; Reset mask for all rows
	LDA #$FF
	LDX #$07
	
ResetRowMask
	STA MASK,X
	DEX
	BPL ResetRowMask
	
ResetDeadEnemyPosition
	LDX TEMP
	TXA	
	AND #$01
	BEQ ResetEnemyLeft
	BNE ResetEnemyRight

CheckEnemyCollision
; Handle collisions
	LDA CXM0P
	AND #$80
	BEQ CheckPlayerCollision
	
; Get the row of the current enemy that was hit
	LDA #NUMPLAYAREALINES-24
	SBC PLAYERMISSILEVERTPOS
	AND #$F0
	LSR
	LSR
	LSR
	LSR
	STA ENEMYHIT
	TAX
		
; Increment row of enemy hit and store it in the temp variable
	INX
	STX TEMPDIGITVALUE

; Turn on decimal mode and add the score of the enemy that was hit
	SED
	CLC
	LDA SCORE3
	ADC TEMPDIGITVALUE
	STA SCORE3
	LDA SCORE2
	ADC #$00
	STA SCORE2
	LDA SCORE1
	ADC #$00
	STA SCORE1
	CLD

	LDA #$00
	STA PLAYERMISSILEVERTPOS
	
	LDA #$0F
	STA DEATHFRAMECOUNT
	
CheckPlayerCollision
; Handle collisions
	LDA CXM1P
	AND #$80
	BEQ SetBackgroundColor

; Disable player and enemy missiles
	LDA #$00
	STA PLAYERMISSILEVERTPOS
	STA ENEMYPLAYERMISSILEVERTPOS
	
; Set game state to "not yet started"
	LDA #$01
	STA GAMESTATE
	
; Set the death frame count
	LDA #$0F
	STA PLAYERDEATHFRAMECOUNT
	DEC PLAYERLIVES
	BEQ SetGameOver
	JMP SetBackgroundColor

SetGameOver
	LDA #$04
	STA GAMESTATE
	
SetBackgroundColor
	LDA #$98
	STA BACKGROUNDCOLOR
	
MoveEnemies
	LDX #NUMENEMYROWS
	JSR GenerateRandom	
	STA TEMPDIGITVALUE

ProcessRow
	CPX ENEMYHIT
	BNE MoveEnemy

	LDA DEATHFRAMECOUNT
	BEQ MoveEnemy
	JMP ProcessNextEnemy

ResetEnemyLeft
	LDA #MAXHORZPOS+6
	STA ENEMYHORZPOS,X
	JMP SetEnemyColor

ResetEnemyRight
	LDA #MINHORZPOS-6
	STA ENEMYHORZPOS,X
	JMP SetEnemyColor
	
MoveEnemy
	TXA
	AND #$01
	BEQ MoveEnemyLeft
	
; Move Enemy Right
	LDA #MAXHORZPOS+6
	CMP ENEMYHORZPOS,X
	BEQ ResetEnemyRight

	INC ENEMYHORZPOS,X
	JMP SetEnemyColor
	
MoveEnemyLeft
	LDA #MINHORZPOS-6
	CMP ENEMYHORZPOS,X
	BEQ ResetEnemyLeft
	
	DEC ENEMYHORZPOS,X
	
SetEnemyColor
	LDA SCORE2
	AND #$F0
	LSR
	LSR
	LSR
	LSR
	TAY
	LDA ShipColors,Y
	STA ENEMYCOLOR,X
	
ProcessNextEnemy
	DEX
	BPL ProcessRow

CheckJoystick
	LDA #$40
	BIT SWCHA		; Left
	BEQ MovePlayerLeft
	
	LDA #$80
	BIT SWCHA		; Right
	BEQ MovePlayerRight
	JMP CheckButton
	
MovePlayerLeft
	LDA PLAYERHORZPOS
	CMP #MINHORZPOS
	BEQ CheckButton
	
	DEC PLAYERHORZPOS
	JMP CheckButton

MovePlayerRight
	LDA PLAYERHORZPOS
	CMP #MAXHORZPOS
	BEQ CheckButton

	INC PLAYERHORZPOS

CheckButton
	STA CXCLR
	LDA INPT4
    BMI ClearButtonFlag
    
; Skip launching missile if button is held down
	LDA BUTTONDOWN
	BNE VerifyGameState	

; Set game state to "active"
	LDA #$00
	STA GAMESTATE

; Flag button as pressed
	LDA #$01
	STA BUTTONDOWN
	
; Make sure a missile isn't launched
	LDA PLAYERMISSILEVERTPOS
	BNE VerifyGameState
	
; Copy current player horizontal position into missile horizontal position, adjusting to center missile over player
	LDA PLAYERHORZPOS
	ADC #$04
	STA PLAYERMISSILEHORZPOS
	
; Set missile's vertical position slightly above player
	LDA #NUMPLAYAREALINES-16
	STA PLAYERMISSILEVERTPOS
	
; Set sound effect counter
	LDA #$E0
	STA SHOOTSOUNDCOUNTER

ClearButtonFlag
; Button released
	LDA #$00
	STA BUTTONDOWN
	
VerifyGameState
	LDA GAMESTATE
	BEQ CheckEnemyMissile
	JMP DoShotSoundEffect

CheckEnemyMissile
	LDA ENEMYPLAYERMISSILEVERTPOS
	BEQ LaunchEnemyMissile
	
	CMP #NUMPLAYAREALINES
	BMI MoveEnemyMissile
	
; Disable enemy missile
	LDA #$00
	STA ENEMYPLAYERMISSILEVERTPOS
	BEQ SetPlayerMissilePos
	
MoveEnemyMissile
	INC ENEMYPLAYERMISSILEVERTPOS
	JMP SetPlayerMissilePos
	
LaunchEnemyMissile
	LDX #$00

CheckEnemyPos
	LDA MASK,X
	BEQ CheckNextEnemyPos
	LDA ENEMYHORZPOS,X
	ADC #04
	CMP PLAYERHORZPOS
	BEQ SetEnemyMissilePos
	
CheckNextEnemyPos
	INX
	CPX #$08
	BEQ SetPlayerMissilePos
	BNE CheckEnemyPos
	
SetEnemyMissilePos
	ADC #$04
	STA ENEMYPLAYERMISSILEHORZPOS
	LDA EnemyVertPos,X
	STA ENEMYPLAYERMISSILEVERTPOS

SetPlayerMissilePos
; Check position of missile
	LDA PLAYERMISSILEVERTPOS
	BEQ DoShotSoundEffect
	
; Change vertical position of missile
	DEC PLAYERMISSILEVERTPOS
	DEC PLAYERMISSILEVERTPOS

DoShotSoundEffect	
; Do the sound effect
	LDA SHOOTSOUNDCOUNTER
	BEQ DisableSound
	
	INC SHOOTSOUNDCOUNTER
	STA AUDF0
	LDA #$0F
	STA AUDC0
	
DisableSound
	STA AUDV0
	
; Copy vertical positions into temp locations
ResetValues
	LDA #LINESFORENEMYROW
	STA ENEMYVERTPOSTEMP
	LDA #NUMENEMYROWS
	STA ENEMYROW
	LDA #$00
	LDA PLAYERMISSILEVERTPOS
	STA PLAYERMISSILEVERTPOSTEMP
	LDA ENEMYPLAYERMISSILEVERTPOS
	STA ENEMYPLAYERMISSILEVERTPOSTEMP

; Set digit pointers
	LDA SCORE3
	JSR CalculateScoreDigitPointer
	STY DIGITPOINTER6
	STX DIGITPOINTER5
	LDA SCORE2
	JSR CalculateScoreDigitPointer
	STY DIGITPOINTER4
	STX DIGITPOINTER3
	LDA SCORE1
	JSR CalculateScoreDigitPointer
	STY DIGITPOINTER2
	STX DIGITPOINTER1
	
WaitForTimer
	LDA INTIM
	BNE WaitForTimer
	STA WSYNC
	STA VBLANK

; Set the background color
	LDA BACKGROUNDCOLOR
	STA COLUBK	
	LDY #$20

; Draw the score
DrawScore
	STA WSYNC
	LDA #$00
	STA GRP0
	STA GRP1    
	STA WSYNC
	NOP
	STA GRP0 
	STA GRP1 
	LDY #$07 
	STY SCORESCANLINE  
	LDA #$03 
	STA NUSIZ0  
	STA NUSIZ1  
	STA VDELP0  
	STA VDELP1  
	LDA #$F0 
	STA HMP0 
	LDA SCORECOLOR
	NOP
	STA RESP0
	STA RESP1
	STA COLUP0  
	STA COLUP1  
	STA WSYNC
	STA HMOVE
	
DrawScoreScanline
	LDY SCORESCANLINE  
	LDA (DIGITPOINTER1),Y 
	STA GRP0 
	STA WSYNC
	LDA (DIGITPOINTER2),Y 
	STA GRP1 
	LDA (DIGITPOINTER3),Y 
	STA GRP0 
	LDA (DIGITPOINTER4),Y 
	STA TEMPDIGITVALUE  
	LDA (DIGITPOINTER5),Y 
	TAX
	LDA (DIGITPOINTER6),Y 
	TAY
	LDA TEMPDIGITVALUE  
	STA GRP1 
	STX GRP0 
	STY GRP1 
	STA GRP0 
	DEC SCORESCANLINE  
	BPL DrawScoreScanline
	STA WSYNC

; Clear out players after drawing score
	LDA #$00 
	STA GRP0 
	STA GRP1 
	STA VDELP0  
	STA VDELP1  
	STA NUSIZ0
	STA NUSIZ1
	LDA #$2E
	STA COLUP0
	LDA #$CE						; 2
	STA COLUPF						; 3
	LDX #NUMENEMYROWS
	STA WSYNC

; Position the first enemy
	LDA ENEMYHORZPOS,X
	LDX #$01
	JSR PositionObject
	STA WSYNC

; Position the enemy's missile	
	LDX #$03
	LDA ENEMYPLAYERMISSILEHORZPOS
	JSR PositionObject
	STA WSYNC
	
; Position the player's missile
	LDX #$02
	LDA PLAYERMISSILEHORZPOS
	JSR PositionObject
	
	LDX #$07
	LDA ENEMYCOLOR,X
	STA COLUP1
	LDA MASK,X
	STA TEMP
	
; Draw 144 lines of the play area (minus to two to adjust for horizontal position above)
	LDY #NUMPLAYAREALINES-2        	; 2
	
DrawPlayArea
	LDA #$F8							; 2 68
	DEC PLAYERMISSILEVERTPOSTEMP		; 5 73
	DEC ENEMYVERTPOSTEMP				; 5 78
	DEC ENEMYPLAYERMISSILEVERTPOSTEMP	; 5 83
	STA WSYNC							; 3 86

	BIT ENEMYVERTPOSTEMP				; 3  3
	BNE CheckMissilePosition			; 2  5
		
	LDX ENEMYVERTPOSTEMP				; 3  8
	LDA Ship,X							; 4 12
	AND TEMP
	STA GRP1							; 3 15
	
	CPX #$00							; 2 17
	BNE CheckMissilePosition			; 2 19

; Position the next enemy and get the ship color
	DEC ENEMYROW						; 5 24
	LDX ENEMYROW						; 3 27
	LDA ENEMYCOLOR,X					; 4 31
	STA COLUP1							; 3 34
	LDA MASK,X
	STA TEMP
	CPX #$00							; 2 36
	BMI CheckMissilePosition			; 3 39
	
	LDX ENEMYROW						; 3 42
	LDA ENEMYHORZPOS,X					; 4 46
	LDX #$01							; 2 48
	JSR PositionObject					; 6 
	DEY									; 2 
	DEY									; 2 

	LDA #LINESFORENEMYROW				; 2 
	STA ENEMYVERTPOSTEMP				; 3 
	
CheckMissilePosition
	LDA #$FC							; 2 41
	BIT PLAYERMISSILEVERTPOSTEMP		; 3 44
	BNE CheckEnemyMissilePosition		; 2 46
	
	LDA #$02							; 2 48

CheckEnemyMissilePosition
	STA ENAM0							; 3 51

	LDA #$FC							; 2 53
	BIT ENEMYPLAYERMISSILEVERTPOSTEMP	; 3 56
	BNE FinishScanline					; 2 58
	
	LDA #$02							; 2 60
	
FinishScanline
	STA ENAM1							; 2 62
	DEY									; 2 64
	BNE DrawPlayArea					; 2 66
	
; Position the player
	LDX #$00
	LDA PLAYERHORZPOS
	JSR PositionObject
	
	LDA #$30						; 2
	STA PF0							; 3

	; Draw player, 8 lines
	LDX #$07						; 2
	
DrawPlayer
	STA WSYNC
	LDA P0Data,X
	STA GRP0
	DEX
	BPL DrawPlayer

;  Draws 8 lines of green border
	INX
	STX GRP0

	LDA #$FF
	STA PF0
	STA PF1
	STA PF2

	STX ENAM0
	STX ENAM1
	LDY #$CE
	
DrawBottom
	STY COLUPF
	STA WSYNC
	DEY
	DEY
	CPY #$BE
	BNE DrawBottom
	
	LDA #OSCAN_TIME
	STA TIM64T
	
; Clear out TIA registers
	LDA #$00
	STA COLUBK
	STA PF0
	STA PF1
	STA PF2
	
	LDA #$02
	STA VBLANK
	
; DrawOverscan (30 scanlines)
WaitForOverscan
	LDA INTIM
	BNE WaitForOverscan
	JMP DrawVerticalSync
	
; Reset the reset variables
SetInitialValues
	LDA #$00
	STA SCORE1
	STA SCORE2
	STA SCORE3
	STA PLAYERMISSILEVERTPOS
	STA ENEMYPLAYERMISSILEVERTPOS
	STA PLAYERDEATHFRAMECOUNT
	STA DEATHFRAMECOUNT
	STA SHOOTSOUNDCOUNTER

; Set the enemy size to default of $FF	
	LDA #$FF
	LDX #$07
SetEnemyMaskDefault
	STA MASK,X
	DEX
	BPL SetEnemyMaskDefault

	LDA #CENTERPOS
	STA PLAYERHORZPOS
	
	LDA #$0F
	STA SCORECOLOR
	
; Set the game status to "not yet started"
	LDA #$01
	STA GAMESTATE
	
; Reset the number of players
	LDA #$03
	STA PLAYERLIVES	
	RTS
	
; Gets the low byte of the pointer for each digit
CalculateScoreDigitPointer:
	TAX
	AND #$0F 
	ASL
	ASL
	ASL
	TAY
	TXA
	AND #$F0 
	LSR
	TAX
	RTS
	
PositionEnemies
; Set the initial enemy positions
	LDX #NUMENEMYROWS
ProcessNextHorizontalValue
	JSR GenerateRandom
	CMP #MAXHORZPOS
	BCS ResetToMax
	CMP #MINHORZPOS
	BCC ResetToMin
	JMP StoreValue
	
ResetToMin
	LDA #MINHORZPOS
	JMP StoreValue
	
ResetToMax
	LDA #MAXHORZPOS
	
StoreValue
	STA ENEMYHORZPOS,X
	DEX
	BPL ProcessNextHorizontalValue
	RTS

; Straight from "Air Sea Battle"
; This is Larry Kaplan's typical LFSR psuedo-random number generator. It was
; derived from an article by Don Lancaster that appeared in BYTE magazine.
; Since this is an 8-bit LFSR the numbers will start to repeat after 255
; iterations.
; Notice that the first time this routine is called the RAND is 0. This
; will cause a tap of D6 which will cause the initial value to be 64d.
; see...
; http://www.ciphersbyritter.com/GLOSSARY.HTM#LinearFeedbackShiftRegister
; http://www.cypressmicro.com/products/umselector/StdUM/prs8/PRS8.htm
GenerateRandom
   LSR RAND
   ROL
   EOR RAND
   LSR
   LDA RAND
   BCS skipTap
   ORA #%01000000
   STA RAND
skipTap
   RTS
   
Ship
	.byte %00000000 ; |        |
	.byte %00011000 ; |   ##   |
	.byte %00011000 ; |   ##   |
	.byte %11111111 ; |########|
	.byte %00000000 ; |        |
	.byte %11111111 ; |########|
	.byte %00011000 ; |   ##   |
	.byte %00011000 ; |   ##   |
	
ShipColors
	.byte $0E ; White
	.byte $1E ; Yellow
	.byte $3E ; Orange
	.byte $44 ; Dark Red
	.byte $50 ; Maroon
	.byte $60 ; Purple
	.byte $B2 ; Green
	.byte $CE ; Light Green
	.byte $FE ; Tan
	.byte $00 ; Black

EnemyVertPos
	.byte $89,$77,$65,$53,$41,$2F,$1D,$0B

P0Data
	.byte %11100111 ; |###  ###|
	.byte %11100111 ; |###  ###|
	.byte %11100111 ; |###  ###|
	.byte %01111110 ; | ###### |
	.byte %00111100 ; |  ####  |
	.byte %00111100 ; |  ####  |
	.byte %00011000 ; |   ##   |
	.byte %00011000 ; |   ##   |
	
	ORG $FF00
	; Zero
	.byte $3C ; |  ####  |
	.byte $66 ; | ##  ## |
	.byte $66 ; | ##  ## |
	.byte $66 ; | ##  ## |
	.byte $66 ; | ##  ## |
	.byte $66 ; | ##  ## |
	.byte $66 ; | ##  ## |
	.byte $3C ; |  ####  |

	; One
	.byte $7E ; | ###### |
	.byte $18 ; |   ##   |
	.byte $18 ; |   ##   |
	.byte $18 ; |   ##   |
	.byte $18 ; |   ##   |
	.byte $38 ; |  ###   |
	.byte $18 ; |   ##   |
	.byte $08 ; |    #   |

	; Two
	.byte $7E ; | ###### |
	.byte $60 ; | ##     |
	.byte $60 ; | ##     |
	.byte $3C ; |  ####  |
	.byte $06 ; |     ## |
	.byte $06 ; |     ## |
	.byte $46 ; | #   ## |
	.byte $3C ; |  ####  |

	; Three
	.byte $3C ; |  ####  |
	.byte $46 ; | #   ## |
	.byte $06 ; |     ## |
	.byte $06 ; |     ## |
	.byte $1C ; |   ###  |
	.byte $06 ; |     ## |
	.byte $46 ; | #   ## |
	.byte $3C ; |  ####  |

	; Four
	.byte $0C ; |    ##  |
	.byte $0C ; |    ##  |
	.byte $7E ; | ###### |
	.byte $4C ; | #  ##  |
	.byte $4C ; | #  ##  |
	.byte $2C ; |  # ##  |
	.byte $1C ; |   ###  |
	.byte $0C ; |    ##  |

	; Five
	.byte $3C ; |  ####  |
	.byte $46 ; | #   ## |
	.byte $06 ; |     ## |
	.byte $06 ; |     ## |
	.byte $3C ; |  ####  |
	.byte $60 ; | ##     |
	.byte $60 ; | ##     |
	.byte $7E ; | ###### |

	; Six
	.byte $3C ; |  ####  |
	.byte $66 ; | ##  ## |
	.byte $66 ; | ##  ## |
	.byte $66 ; | ##  ## |
	.byte $7C ; | #####  |
	.byte $60 ; | ##     |
	.byte $62 ; | ##   # |
	.byte $3C ; |  ####  |

	; Seven
	.byte $30 ; |  ##    |
	.byte $30 ; |  ##    |
	.byte $30 ; |  ##    |
	.byte $18 ; |   ##   |
	.byte $0C ; |    ##  |
	.byte $06 ; |     ## |
	.byte $42 ; | #    # |
	.byte $3E ; |  ##### |

	; Eight
	.byte $3C ; |  ####  |
	.byte $66 ; | ##  ## |
	.byte $66 ; | ##  ## |
	.byte $66 ; | ##  ## |
	.byte $3C ; |  ####  |
	.byte $66 ; | ##  ## |
	.byte $66 ; | ##  ## |
	.byte $3C ; |  ####  |

	; Nine
	.byte $3C ; |  ####  |
	.byte $46 ; | #   ## |
	.byte $06 ; |     ## |
	.byte $3E ; |  ##### |
	.byte $66 ; | ##  ## |
	.byte $66 ; | ##  ## |
	.byte $66 ; | ##  ## |
	.byte $3C ; |  ####  |
	
	org $FFEB
	dc.b "(c)2015 JunkoSoft"
	.word Start
	.word Start